
/* ***************************************************************************
    NWDRシステム情報:コンストラクタ関数
**************************************************************************** */
function NwdrSysInfo(
    nCamTotalNum,
    n360MicNum,
    nCamNumH264_1,
    nCamNumH264_2,
    nCamNumH264_3,
    nCamNumJpeg_1,
    nCamNumJpeg_2,
    nCamNumJpeg_3,
    nCamNumMpg4_1,
    nCamNumFace_1,
    lngHbrCamNum,
    sysRecDataTB,
    sysCalcMbps,
    sysDispMbps,
    sysTotalIps,
    extUnitNum,
    mainHddNum,
    intHDDq,
    sysLicNum,
    nwdrModelName,
    lngLastHDDCapa,
    pankflg) {
    this.nCamTotalNum = nCamTotalNum; // 総カメラ数
    this.n360MicNum = n360MicNum; // 総マイク数
    this.nCamNumH264_1 = nCamNumH264_1; // H264(1)
    this.nCamNumH264_2 = nCamNumH264_2; // H264(2)
    this.nCamNumH264_3 = nCamNumH264_3; // H264(3)
    this.nCamNumJpeg_1 = nCamNumJpeg_1; // JPEG(1)
    this.nCamNumJpeg_2 = nCamNumJpeg_2; // JPEG(2)
    this.nCamNumJpeg_3 = nCamNumJpeg_3; // JPEG(3)
    this.nCamNumMpg4_1 = nCamNumMpg4_1; // MPEG4(1)
    this.nCamNumFace_1 = nCamNumFace_1; // H264(顔照合)(1)
    this.lngHbrCamNum = lngHbrCamNum; // 高ビットレート(HighBitRate)カメラ数
    this.sysRecDataTB = sysRecDataTB; // 総容量(NWDR録画済容量)[TB]
    this.sysCalcMbps = sysCalcMbps; // 総録画ビットレート(計算用)
    this.sysDispMbps = sysDispMbps; // 総録画ビットレート(表示用)
    this.sysTotalIps = sysTotalIps; // 総録画ips(JPEG)
    this.extUnitNum = extUnitNum; // 増設ユニット台数
    this.mainHddNum = mainHddNum; // 内蔵HDD数
    this.intHDDq = intHDDq; // HDD数(過去互換性のため)
    this.sysLicNum = sysLicNum; // ライセンス数
    this.nwdrModelName = nwdrModelName; // 派生品番
    this.lngLastHDDCapa = lngLastHDDCapa; // 最終(本体or増設)HDD割当容量[TB]
    this.pankflg = pankflg; // 割り当て不可フラグ
}

/* ***************************************************************************
    NWDRシステム情報プロトタイプ拡張:コピー(clone)
**************************************************************************** */
Array.prototype.clone = function() {
    return Array.apply(null, this);
};

/* ***************************************************************************
    NWDR情報初期化 (constructor)

    tmpRecNum    NWDR台数
**************************************************************************** */
function NWDRArrayInit(tmpRecNum) {
    // パラメータをすべて0にする
    gNwdrInfo.length = 0;
    for (var idx = 0; idx < tmpRecNum; idx++) {
        gNwdrInfo[idx] = new NwdrSysInfo(
            0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
            0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
    }
}

/* ***************************************************************************
    カメラ諸元情報:コンストラクタ関数
**************************************************************************** */
function CamSpecInfo(Mbps, typeCode, modelName) {
    this.Mbps = Mbps;
    this.typeCode = typeCode;
    this.modelName = modelName; // 2028_63164
}

/* ***************************************************************************
    録画容量計算

    nKeyCode     キー入力コード
**************************************************************************** */
function calcdata(nKeyCode) {
    g_strmsg = ''; // メッセージ初期化
    gTotalCamNum = 0; // カメラ総台数初期化
    gTotalMicNum = 0; // マイク総台数初期化

    bND400_4kFirm = false; // 専用ファーム対応

    // 録画日数
    gRecDay = Number(frmNWDR.txtRecDays.value);
    // 録画時間
    gRecHour = Number(frmNWDR.selRecHour.options[frmNWDR.selRecHour.selectedIndex].value);
    // HDD容量
    gSelHddUnitGB = Number(frmNWDR.sel_HDD.options[frmNWDR.sel_HDD.selectedIndex].value);
    // RAID構成
    gSelRaidMode = Number(frmNWDR.sel_RAID.options[frmNWDR.sel_RAID.selectedIndex].value);

    // キーコードが矢印左右の場合は計算しない
    if (nKeyCode == 37 || nKeyCode == 39) {
        return;
    }

    // 日数が0の時は、計算結果を表示せず終了
    if (gRecDay == 0) {
        uiAllResultGrayOut();
        return;
    }

    // カメラ種別取得
    gCamModelNameUI[H264_1] = Number(frmNWDR.selModelH264_1.selectedIndex); //H.264(1)
    gCamModelNameUI[H264_2] = Number(frmNWDR.selModelH264_2.selectedIndex); //H.264(2)
    gCamModelNameUI[H264_3] = Number(frmNWDR.selModelH264_3.selectedIndex); //H.264(3)
    gCamModelNameUI[JPEG_1] = Number(frmNWDR.selModelJpeg_1.selectedIndex); //JPEG(1)
    gCamModelNameUI[JPEG_2] = Number(frmNWDR.selModelJpeg_2.selectedIndex); //JPEG(2)
    gCamModelNameUI[JPEG_3] = Number(frmNWDR.selModelJpeg_3.selectedIndex); //JPEG(3)
    gCamModelNameUI[MPG4_1] = Number(frmNWDR.selModelMpg4_1.selectedIndex); //MPEG4
    gCamModelNameUI[H264FM] = 0; //H.264(顔照合)

    // カメラ台数取得
    gCamNumberUI[H264_1] = Number(frmNWDR.txtCamNumH264_1.value); //H.264(1)
    gCamNumberUI[H264_2] = Number(frmNWDR.txtCamNumH264_2.value); //H.264(2)
    gCamNumberUI[H264_3] = Number(frmNWDR.txtCamNumH264_3.value); //H.264(3)
    gCamNumberUI[JPEG_1] = Number(frmNWDR.txtCamNumJpeg_1.value); //JPEG(1)
    gCamNumberUI[JPEG_2] = Number(frmNWDR.txtCamNumJpeg_2.value); //JPEG(2)
    gCamNumberUI[JPEG_3] = Number(frmNWDR.txtCamNumJpeg_3.value); //JPEG(3)
    gCamNumberUI[MPG4_1] = Number(frmNWDR.txtCamNumMpg4_1.value); //MPEG4
    gCamNumberUI[H264FM] = Number(frmNWDR.txtCamNumFace_1.value); //H.264(顔照合)

    // カメラ台数計算上限例外処理
    for (var i = 0; i < CAM_TYPENUM_ALL; i++) { // カメラ＋全方位マイク総数を取得
        if (gCamNumberUI[i] == '') { // 入力欄が空だったときは
            gCamNumberUI[i] = 0;     // 台数を0にする
        }

        gTotalCamNum = gTotalCamNum + Number(gCamNumberUI[i]); // 総カメラ台数に加算
        // 全方位マイクの場合
        if (camKind_H264_360SMR == gCamModelNameUI[i] || camKind_H264_9MSMR == gCamModelNameUI[i]) {
            gTotalMicNum += Number(gCamNumberUI[i]);
        }
    }
    // 総カメラ台数が0の時は、計算結果を表示せず終了
    if (gTotalCamNum == 0) {
        uiAllResultGrayOut();
        return;
    }
    // 総台数が1000より大きい場合は、メッセージを表示し計算せず終了
    if ((gTotalCamNum + gTotalMicNum) > 1000) {
        uiMsgDialogPopup(CAMNUMOVER, ALARTMSGMODE);
        uiAllResultGrayOut();
        return;
    }

    // 計算結果欄を初期化
    uiAllResultGrayOut();
    uiAllResultInit();

    // ■■■STEP1-1■■■
    // 各カメラのビットレートをUIより取得。単位が異なることに注意
    // JPEG -> [Mbps]
    // H.264, MPEG4 -> [kbps]
    gBitRateUI[H264_1] = Number(frmNWDR.selKbpsH264_1.value); //H.264(1)
    gBitRateUI[H264_2] = Number(frmNWDR.selKbpsH264_2.value); //H.264(2)
    gBitRateUI[H264_3] = Number(frmNWDR.selKbpsH264_3.value); //H.264(3)
//    gBitRateUI[JPEG_1] = Number(frmNWDR.txtMbpsJpeg_1.value); //JPEG(1)
//    gBitRateUI[JPEG_2] = Number(frmNWDR.txtMbpsJpeg_2.value); //JPEG(2)
//    gBitRateUI[JPEG_3] = Number(frmNWDR.txtMbpsJpeg_3.value); //JPEG(3)
    gBitRateUI[MPG4_1] = Number(frmNWDR.selKbpsMpg4_1.value); //MPEG4
    gBitRateUI[H264FM] = Number(frmNWDR.selKbpsFace_1.value); //H.264(顔照合)

    // ここからは、まずJPEGのゲタを含めた、帯域ビットレートを扱う
    //Advanced VBR対応
    var coefAVBR_1 = Number(frmNWDR.selPrioH264_1.value);
    var coefAVBR_2 = Number(frmNWDR.selPrioH264_2.value);
    var coefAVBR_3 = Number(frmNWDR.selPrioH264_3.value);

   // AVBR係数決定. 高(sel:2)->4, 中(sel:3)->3, 低(sel:4)->2, その他->1 へ解釈
    coefAVBR_1 = ((coefAVBR_1) > 1 ? (6 - coefAVBR_1) : 1);
    coefAVBR_2 = ((coefAVBR_2) > 1 ? (6 - coefAVBR_2) : 1);
    coefAVBR_3 = ((coefAVBR_3) > 1 ? (6 - coefAVBR_3) : 1);

    // ====================================================================
    // 表示用ビットレート(gDispMbps)設定
    // ====================================================================
    gDispMbps[H264_1] = gBitRateUI[H264_1] * coefAVBR_1 / 1024; //H.264(1)
    gDispVideoMbps[H264_1] = gDispMbps[H264_1];
    gDispMbps[H264_2] = gBitRateUI[H264_2] * coefAVBR_2 / 1024; //H.264(2)
    gDispVideoMbps[H264_2] = gDispMbps[H264_2];
    gDispMbps[H264_3] = gBitRateUI[H264_3] * coefAVBR_3 / 1024; //H.264(3)
    gDispVideoMbps[H264_3] = gDispMbps[H264_3];
    gDispMbps[JPEG_1] = gDispBandJpegMbps[0];    //JPEG(1)
    gDispVideoMbps[JPEG_1] = gDispMbps[JPEG_1];
    gDispMbps[JPEG_2] = gDispBandJpegMbps[1];    //JPEG(2)
    gDispVideoMbps[JPEG_2] = gDispMbps[JPEG_2];
    gDispMbps[JPEG_3] = gDispBandJpegMbps[2];    //JPEG(3)
    gDispVideoMbps[JPEG_3] = gDispMbps[JPEG_3];
    gDispMbps[MPG4_1] = gBitRateUI[MPG4_1] / 1024; //MPEG4
    gDispVideoMbps[MPG4_1] = gDispMbps[MPG4_1];
    gDispMbps[H264FM] = gBitRateUI[H264FM] / 1024; //H.264(顔照合)
    gDispVideoMbps[H264FM] = gDispMbps[H264FM];

    // ====================================================================
    // 計算用ビットレート(gCalcMbps)設定
    // ====================================================================
    gCalcMbps[H264_1] = gBitRateUI[H264_1] / 1000; //H.264(1)
    gCalcMbps[H264_2] = gBitRateUI[H264_2] / 1000; //H.264(2)
    gCalcMbps[H264_3] = gBitRateUI[H264_3] / 1000; //H.264(3)
    gCalcMbps[JPEG_1] = gCalcBandJpegMbps[0];       //JPEG(1)
    gCalcMbps[JPEG_2] = gCalcBandJpegMbps[1];       //JPEG(2)
    gCalcMbps[JPEG_3] = gCalcBandJpegMbps[2];       //JPEG(3)
    gCalcMbps[MPG4_1] = gBitRateUI[MPG4_1] / 1000; //MPEG4
    gCalcMbps[H264FM] = gBitRateUI[H264FM] / 1000; //H.264(顔照合)

    // ====================================================================
    // 録画容量計算用ビットレート設定(JPEG)
    // ====================================================================
    gCalcJpegMbps[0] = gCalcRecJpegMbps[0]; //JPEG(1)
    gCalcJpegMbps[1] = gCalcRecJpegMbps[1]; //JPEG(2)
    gCalcJpegMbps[2] = gCalcRecJpegMbps[2]; //JPEG(3)

    // ====================================================================
    // 映像ビットレート設定
    // ====================================================================
    gVideoMbps[H264_1] = gCalcMbps[H264_1] * coefAVBR_1; //H.264(1)
    gVideoMbps[H264_2] = gCalcMbps[H264_2] * coefAVBR_2; //H.264(2)
    gVideoMbps[H264_3] = gCalcMbps[H264_3] * coefAVBR_3; //H.264(3)
    gVideoMbps[JPEG_1] = gCalcJpegMbps[0]; //JPEG(1)
    gVideoMbps[JPEG_2] = gCalcJpegMbps[1]; //JPEG(2)
    gVideoMbps[JPEG_3] = gCalcJpegMbps[2]; //JPEG(3)
    gVideoMbps[MPG4_1] = gCalcMbps[MPG4_1]; //MPEG4
    gVideoMbps[H264FM] = gCalcMbps[H264FM]; //H.264(顔照合)

    // ====================================================================
    // JPEG録画IPS数設定 (H.264/MPEG4の場合は、当然0)
    // ====================================================================
    gJpegIps[H264_1] = 0; //H.264(1)
    gJpegIps[H264_2] = 0; //H.264(2)
    gJpegIps[H264_3] = 0; //H.264(3)
    gJpegIps[JPEG_1] = Number(frmNWDR.selFpsJpeg_1.value); //JPEG(1)
    gJpegIps[JPEG_2] = Number(frmNWDR.selFpsJpeg_2.value); //JPEG(2)
    gJpegIps[JPEG_3] = Number(frmNWDR.selFpsJpeg_3.value); //JPEG(3)
    gJpegIps[MPG4_1] = 0; //MPEG4
    gJpegIps[H264FM] = 0; //H.264(顔照合)
    // ====================================================================

    // ■■■STEP1-3■■■
    // VMD/音声加算フラグ
    // 1 : 両方とも加算なし, 2 : 片方だけ加算あり, 3 : 両方とも加算あり
    var intcheck = 1;

    // VMD情報にチェックがついていたら、加算する
    if (frmNWDR.chkvmd.checked == true) {
        AddVmdBitrate(); // VMD情報加算
        intcheck++; // フラグ変更
        g_DynaConnect[nNwdrNV300] = defCamNumMax[nNwdrNV300]; // 接続最大数設定
    }

    // 360マイク音声計算対応
    Add360MicAudioBitrate();

    // 音声情報にチェックがついていたら、加算する
    if (frmNWDR.chkaudio.checked == true) {
        AddAudioBitrate(); // 音声情報加算
        intcheck++; // フラグ変更
        g_DynaConnect[nNwdrNV300] = defCamNumMax[nNwdrNV300]; // 接続最大数設定
    }

    // フラグをみて、ND400の接続最大数を決める
    if (intcheck > 1) { // VMD・音声どちらかが有効な場合
        // ND400は両方対応。VMD/音声のセッション数を足して最大接続数を決める
        if (((gCamNumberUI[H264_1] != 0) || (gCamNumberUI[H264_2] != 0) || (gCamNumberUI[H264_3] != 0)) && //H.264(1)-(3)
                (gCamNumberUI[JPEG_1] == 0) && (gCamNumberUI[JPEG_2] == 0) && (gCamNumberUI[JPEG_3] == 0) && //JPEG(1)-(3)
                (gCamNumberUI[MPG4_1] == 0) && (gCamNumberUI[H264FM] == 0)) { //MPEG4, H.264(顔照合)
            // Advancedモード有効時 ⇒ 96ch
            g_DynaConnect[nNwdrND400] = defMaxConAddInfoAdv[nNwdrND400] / intcheck;
        } else {
            // Advancedモード無効時 ⇒ 72ch
            g_DynaConnect[nNwdrND400] = defMaxConAddInfo[nNwdrND400] / intcheck;
        }
    } else { // VMD・音声どちらかも無効の場合
        // VMD・音声両方OFFの場合は登録可能カメラ台数がそのまま接続可能数になる
        g_DynaConnect[nNwdrND400] = defCamNumMax[nNwdrND400];
        g_DynaConnect[nNwdrNV300] = defCamNumMax[nNwdrNV300];
    }
    // ND400の場合、VMD・音声両方有効なとき接続可能数は 1/3になるので、割り切れなかった数を切り捨てる
    g_DynaConnect[nNwdrND400] = Math.floor(g_DynaConnect[nNwdrND400]);

    // ■■■STEP1-2■■■
    // --------------------------------
    // 総ビットレート(表示用)
    // --------------------------------
    var totalDispMbps = 0;
    for (var idx = 0; idx < CAM_TYPENUM_ALL; idx++) {
        // 表示用
        totalDispMbps += (gCamNumberUI[idx] * gDispMbps[idx]);
    }

    // 総ビットレート表示[Mbps]
    frmNWDR.txttotalbit.value = NumberToFixed(totalDispMbps, 3);

    // ここまではJPEGのゲタを含めた、帯域ビットレートを扱ってきた
    // ここからは録画計算ビットレートにする
    gCalcMbps[JPEG_1] = gCalcJpegMbps[0]; //JPEG(1)
    gCalcMbps[JPEG_2] = gCalcJpegMbps[1]; //JPEG(2)
    gCalcMbps[JPEG_3] = gCalcJpegMbps[2]; //JPEG(3)

    // 各カメラ毎の録画データ量を算出する
    gCamModelParamInfo.length = 0;
    for (var idx = 0; idx < CAM_TYPENUM_ALL; idx++) {
        // 録画容量計算
        var each_size =
            ((gCalcMbps[idx] / 8) * 60 * 60 * (gRecHour * gRecDay)) / 1000 / 1000;

        // 録画容量を変数に格納する
        gCamModelParamInfo.push({
            'camType': [idx],
            'camNum': gCamNumberUI[idx],
            'camCalcBR': gCalcMbps[idx],
            'camJpgIps': gJpegIps[idx],
            'camRecTB': GetMarumCapa2(each_size),
            'camDispBR': gDispMbps[idx],
            'camModel': gCamModelNameUI[idx],
            'camDispVBR': gDispVideoMbps[idx]
        });
    }

    // --------------------------------
    // 総ビットレート(計算用)
    // --------------------------------
    var totalCalcMbps = 0; // ビットレート合計値
    for (var idx = 0; idx < CAM_TYPENUM_ALL; idx++) {
        totalCalcMbps += (gCamNumberUI[idx] * gCalcMbps[idx]); // 計算用
    }

    // 総録画容量算出
    var totalRecDataTB =
        ((totalCalcMbps / 8) * 60 * 60 * (gRecHour * gRecDay)) / 1000 / 1000;

    // 計算結果(理論値)-総録画容量表示[TB]
    frmNWDR.txttotalcap.value = GetMarumCapa(totalRecDataTB);

    // ===============================================================
    // 各NWDR毎の結果を計算する
    // ===============================================================
    // 録画条件により計算不要なNWDRを特定する
    SetDisplayResultFlag_InputParam();

    // -----------------------------------------
    // ◆ND400の結果を計算する◆
    // -----------------------------------------
    if (1 == g_DsplayResultFlag[nNwdrND400]) {
        // ND400の吸い込み上限ビットレートを超えているものがないか
        var chLimOK = true;
        for (var idx = 0; idx < CAM_TYPENUM_ALL; idx++)
        {
          if (0 < gCamNumberUI[idx] && (ND400_LIM_MBPS < gDispVideoMbps[idx]))
          {
              chLimOK = false;
              break;
          }
        }

        // (1)容量計算実行
        var buf = -1;
        if (chLimOK)
        {
            buf = basicNwdrCalc(nNwdrND400);
        }
        if (buf < 0) {
            dataND400out(); // 録画不可
        } else {
            // (3-1)計算結果出力
            ND400_output(nNwdrND400);
            // (3-2)詳細情報出力
            if (frmNWDR.chkdetail.checked == true) {
                DetailOutput(gNwdrInfo, nNwdrND400, '');
            }
        }
    }

    // -----------------------------------------
    // ◆NV300の結果を計算する◆
    // -----------------------------------------
    if (1 == g_DsplayResultFlag[nNwdrNV300]) {
        // NV300の吸い込み上限ビットレートを超えているものがないか
        var chLimOK = true;
        for (var idx = 0; idx < CAM_TYPENUM_ALL; idx++)
        {
          if (0 < gCamNumberUI[idx] && (NV300_LIM_MBPS < gDispVideoMbps[idx]))
          {
              chLimOK = false;
              break;
          }
        }
        if (chLimOK)
        {
            if (isJpMirror()) { // JP、かつ Mirrorの場合
                // (1)容量計算実行
                nv300CalcJP();
                nv300_lowprice = nv300SysCost[lp];
                // (3-1)計算結果出力
                NV300_output(nNwdrNV300);
                // (3-2)詳細情報出力
                if (frmNWDR.chkdetail.checked == true) {
                    DetailOutput(gNwdrInfo, nNwdrNV300, '');
                }
            } else { //JP Mirror以外の全て
                // (1)容量計算実行
                var buf = basicNwdrCalc(nNwdrNV300);
                if (buf < 0) {
                    dataNV300out(); // 録画不可
                } else {
                    // (3-1)計算結果出力
                    NV300_output(nNwdrNV300);

                    // (3-2)詳細情報出力
                    if (frmNWDR.chkdetail.checked == true) {
                        DetailOutput(gNwdrInfo, nNwdrNV300, '');
                    }
                }
            }
        }
        else
        {
            dataNV300out(); // 録画不可
        }
    }
}

/* ***************************************************************************
    録画条件から、NWDR毎の計算結果表示を制御する
**************************************************************************** */
function SetDisplayResultFlag_InputParam() {
    // 表示フラグ初期化
    g_DsplayResultFlag[nNwdrND400] = 1;
    g_DsplayResultFlag[nNwdrNV300] = ('zh' == app_dst ? 0 : 1); // NV300->zh非対応

    //--------------------------------------------
    // ●H.264●
    //--------------------------------------------
    var kindidx = 0;
    var minidx = 0;
    var maxidx = CAM_TYPENUM_H264;
    for (; kindidx < maxidx; kindidx++) {
        // オブジェクト変数
        var objParts = new Array();
        // 設定するオブジェクトを取得
        uiGetH264Object((kindidx - minidx + 1).toString(), objParts);

        // 該当ビットレートテーブルを読み込む
        var tblBitrate =
            camBitrate_H264[
                objParts[0].selectedIndex][     // 0: モデル名(model)
                objParts[2].selectedIndex][     // 1: 撮像モード(scan)
                objParts[3].selectedIndex][     // 2: 優先モード(prio)
                objParts[1].selectedIndex][     // 3: 解像度(reso)
                objParts[4].selectedIndex][     // 4: 画質(qual)
                objParts[5].selectedIndex + 1]; // 5: ビットレート情報

        // カメラ台数が0以外のときに、対応チェックを実施
        if (0 < gCamNumberUI[kindidx]) {
            g_DsplayResultFlag[nNwdrND400] &= tblBitrate[4];  // ND400
            g_DsplayResultFlag[nNwdrNV300] &= tblBitrate[7];  // NV300
        }
    }

    //--------------------------------------------
    // ●JPEG●
    //--------------------------------------------
    minidx = maxidx;
    maxidx += CAM_TYPENUM_JPEG;
    for (; kindidx < maxidx; kindidx++) {
        // オブジェクト変数
        var objParts = new Array();
        // 設定するオブジェクトを取得
        uiGetJpegObject((kindidx - minidx + 1).toString(), objParts);

        // 該当画質テーブル
        var tblQuality =
            camQuality_Jpeg[
                objParts[0].selectedIndex][
                objParts[2].selectedIndex][
                objParts[1].selectedIndex][
                objParts[3].selectedIndex + 1];

        // カメラ台数が0以外のときに、対応チェックを実施
        if (0 < gCamNumberUI[kindidx]) {
            // ◆ND400
            g_DsplayResultFlag[nNwdrND400] &= tblQuality[4];  // ND400

            // ◆ND300, ND200に対応しているか(未使用)
            // i-PROのFHDは画質5～9、5ips以下の場合のみ
            var recfps = Number(
                objParts[4].options[objParts[4].selectedIndex].value);
            var recfps_text =
                objParts[4].options[objParts[4].selectedIndex].text;

            // ◆NV300に対応しているか
            g_DsplayResultFlag[nNwdrNV300] &= tblQuality[7];  // NV300
            if (0 <= recfps_text.indexOf(s_mark_notice)) {
                g_DsplayResultFlag[nNwdrNV300] = 0;
            }
        }
    }

    //--------------------------------------------
    // ●MPEG4●
    //--------------------------------------------
    minidx = maxidx;
    maxidx += CAM_TYPENUM_MPG4;
    for (; kindidx < maxidx; kindidx++) {
        // オブジェクト変数
        var objParts = new Array();
        // 設定するオブジェクトを取得
        uiGetMpeg4Object((kindidx - minidx + 1).toString(), objParts);

        // 該当ビットレートテーブル
        var tblBitrate =
            camBitrate_Mpeg4[
                objParts[0].selectedIndex][     // 0: モデル名(model)
                objParts[2].selectedIndex][     // 1: 撮像モード(scan)
                objParts[3].selectedIndex][     // 2: 優先モード(prio)
                objParts[1].selectedIndex][     // 3: 解像度(reso)
                objParts[4].selectedIndex][     // 4: 画質(qual)
                objParts[5].selectedIndex + 1]; // 5: ビットレート情報

        // カメラ台数が0以外のときに、対応チェックを実施
        if (0 < gCamNumberUI[kindidx]) {
            g_DsplayResultFlag[nNwdrND400] &= tblBitrate[4];  // ND400
            g_DsplayResultFlag[nNwdrNV300] = 0;               // NV300
        }
    }

    //--------------------------------------------
    // ●H.264(顔照合):NV300のみ対応●
    //--------------------------------------------
    if (0 < gCamNumberUI[kindidx]) {
        g_DsplayResultFlag[nNwdrND400] = 0;
    }

    //--------------------------------------------
    // ●HDD容量●
    //--------------------------------------------
    switch (gSelHddUnitGB) {
        case 500:
        case 1500:
            g_DsplayResultFlag[nNwdrNV300] = 0; // ND400のみ対応
            break;

        case 1000:
        case 2000:
        case 3000:
        case 4000:
        case 5000:
        case 6000:
            break;

        default: // その他は全て非対応
            g_DsplayResultFlag[nNwdrND400] = 0;
            g_DsplayResultFlag[nNwdrNV300] = 0;
            break;
    }

    //--------------------------------------------
    // ●RAID●
    //--------------------------------------------
    switch (gSelRaidMode) {
        case 0: // Single
        case 5 : // RAID5
        case 6 : // RAID6
            break; // 対応

        case 1 : // RAID1
            g_DsplayResultFlag[nNwdrND400] = 0; // NV300のみ対応
            break;

        default: // その他は全て非対応
            g_DsplayResultFlag[nNwdrND400] = 0;
            g_DsplayResultFlag[nNwdrNV300] = 0;
            break;
    }

    //--------------------------------------------
    // ●カメラVMD機能との組み合わせを調べる●
    //--------------------------------------------
    if (true == frmNWDR.chkvmd.checked) {
        SetDsplayResultByCamVmd();
    }

    //--------------------------------------------
    // ●カメラ音声機能との組み合わせを調べる●
    //--------------------------------------------
    if (true == frmNWDR.chkaudio.checked) {
        SetDsplayResultByCamAudio();
    }

    //--------------------------------------------
    // ●レコーダーVMD/音声対応は、接続可能数として既に調査済み●
    //--------------------------------------------
    for (var idx = 0; idx < g_DynaConnect.length; idx++) {
        if (0 == g_DynaConnect[idx]) {
            g_DsplayResultFlag[idx] = 0;
        }
    }
}

/* ***************************************************************************
    カメラのVMD対応状況によって表示フラグを設定する
**************************************************************************** */
function SetDsplayResultByCamVmd() {

    // ●H.264●
    var kindidx = 0;
    var maxidx = CAM_TYPENUM_H264;
    for (; kindidx < maxidx; kindidx++) {
        // カメラ台数が0以外のときに、対応チェックを実施
        if (0 < gCamNumberUI[kindidx]) {
            // BB-HCM7**はVMD録画不可
            if (camKind_H264_Hcm7 == gCamModelNameUI[kindidx]) {
                g_DsplayResultFlag[nNwdrND400] = 0;
                g_DsplayResultFlag[nNwdrNV300] = 0;
            }
        }
    }

    // ●JPEG●
    maxidx += CAM_TYPENUM_JPEG;
    for (; kindidx < maxidx; kindidx++) {
        // カメラ台数が0以外のときに、対応チェックを実施
        if (0 < gCamNumberUI[kindidx]) {
            // BB-HCMはVMD録画不可
            if (camKind_Jpeg_Hcm7 == gCamModelNameUI[kindidx] || camKind_Jpeg_Hcm5 == gCamModelNameUI[kindidx]) {
                g_DsplayResultFlag[nNwdrND400] = 0;
                g_DsplayResultFlag[nNwdrNV300] = 0;
            }
        }
    }

    // ●MPEG4●
    maxidx += CAM_TYPENUM_MPG4;
    for (; kindidx < maxidx; kindidx++) {
        // カメラ台数が0以外のときに、対応チェックを実施
        if (0 < gCamNumberUI[kindidx]) {
            // BB-HCM7**はVMD録画不可
            if (camKind_Mpeg4_Hcm7 == gCamModelNameUI[kindidx]) {
                g_DsplayResultFlag[nNwdrND400] = 0;
                g_DsplayResultFlag[nNwdrNV300] = 0;
            }
        }
    }

    // H.264(顔照合)はVMD対応
}


/* ***************************************************************************
    VMD情報を加算する
**************************************************************************** */
function AddVmdBitrate() {
    // H.264の種別によってVMD情報を加算するかどうか決める
    var kindidx = 0;
    var minidx = 0;
    var maxidx = CAM_TYPENUM_H264;
    for (; kindidx < maxidx; kindidx++) {
        // BB-HCM7**以外はVMDサイズを加算する
        if (camKind_H264_Hcm7 != gCamModelNameUI[kindidx]) {
            gCalcMbps[kindidx] += Number(CALC_VMDINFO); // 計算用ビットレート加算
            gDispMbps[kindidx] += Number(DISP_VMDINFO); // 表示用ビットレート加算
        }
    }

    // ●JPEGの種別によってVMD情報を加算するかどうか決める●
    minidx = maxidx;
    maxidx += CAM_TYPENUM_JPEG;
    for (; kindidx < maxidx; kindidx++) {
        // BB-HCM以外はVMDサイズを加算する
        if (camKind_Jpeg_Hcm7 != gCamModelNameUI[kindidx] && camKind_Jpeg_Hcm5 != gCamModelNameUI[kindidx]) {
            // 計算用ビットレート加算
            gCalcMbps[kindidx] += Number(CALC_VMDINFO);
            gCalcJpegMbps[kindidx - minidx] += Number(CALC_VMDINFO);
            // 表示用ビットレート加算
            gDispMbps[kindidx] += Number(DISP_VMDINFO);
        }
    }

    // ●MPEG4の種別によってVMD情報を加算するかどうか決める●
    minidx = maxidx;
    maxidx += CAM_TYPENUM_MPG4;
    for (; kindidx < maxidx; kindidx++) {
        // BB-HCM7**以外はVMDサイズを加算する
        if (camKind_Mpeg4_Hcm7 != gCamModelNameUI[kindidx]) {
            gCalcMbps[kindidx] += Number(CALC_VMDINFO); // 計算用ビットレート加算
            gDispMbps[kindidx] += Number(DISP_VMDINFO); // 表示用ビットレート加算
        }
    }

    // ●H.264(顔照合は必ずVMD情報を加算する)●
    minidx = maxidx;
    maxidx += CAM_TYPENUM_FACE;
    for (; kindidx < maxidx; kindidx++) {
        gCalcMbps[kindidx] += Number(CALC_VMDINFO); // 計算用ビットレート加算
        gDispMbps[kindidx] += Number(DISP_VMDINFO); // 表示用ビットレート加算
    }
}

/* ***************************************************************************
    カメラのオーディオ対応状況によって表示フラグを設定する
**************************************************************************** */
function SetDsplayResultByCamAudio() {
    // ●H.264●
    var kindidx = 0;
    var maxidx = CAM_TYPENUM_H264;
    for (; kindidx < maxidx; kindidx++) {
        // 音声非対応のカメラは現状ない
    }

    // ●JPEG●
    maxidx += CAM_TYPENUM_JPEG;
    for (; kindidx < maxidx; kindidx++) {
        // カメラ台数が0以外のときに、対応チェックを実施
        if (0 < gCamNumberUI[kindidx]) {
            // BB-HCM5,3,1は録音不可
            if (camKind_Jpeg_Hcm5 == gCamModelNameUI[kindidx]) {
                g_DsplayResultFlag[nNwdrND400] = 0;
                g_DsplayResultFlag[nNwdrNV300] = 0;
            }
        }
    }

    // ●MPEG4●
    maxidx += CAM_TYPENUM_MPG4;
    for (; kindidx < maxidx; kindidx++) {
        // 音声非対応のカメラは現状ない
    }

    // H.264(顔照合)はVMD対応
}

/* ***************************************************************************
    全方位マイク音声情報を加算する
**************************************************************************** */
function Add360MicAudioBitrate() {
    var kindidx = 0;
    var maxidx = CAM_TYPENUM_H264;
    for (var kindidx = 0; kindidx < CAM_TYPENUM_H264; kindidx++) {
        if (camKind_H264_360SMR == gCamModelNameUI[kindidx] || camKind_H264_9MSMR == gCamModelNameUI[kindidx]) { // 全方位マイクの場合
            // 計算用ビットレート加算
            gCalcMbps[kindidx] += Number(CALC_AUDIO_360);
            // 表示用ビットレート加算
            gDispMbps[kindidx] += Number(DISP_AUDIO_360);
            gVideoMbps[kindidx] += Number(DISP_AUDIO_360); // 全方位マイク音声部を表示にも考慮 2028_62274
        }
    }
}

/* ***************************************************************************
    音声情報を加算する
**************************************************************************** */
function AddAudioBitrate() {
    // H.264はすべて音声対応
    var kindidx = 0;
    var minidx = 0;
    var maxidx = CAM_TYPENUM_H264;
    for (; kindidx < maxidx; kindidx++) {
        if (camKind_H264_360SMR != gCamModelNameUI[kindidx] || camKind_H264_9MSMR != gCamModelNameUI[kindidx]) { // 全方位マイク紐付け以外の場合
            // 計算用ビットレート加算
            gCalcMbps[kindidx] += Number(CALC_AUDIO) + Number(AUDIOADDDATA);
            // 表示用ビットレート加算
            gDispMbps[kindidx] += Number(DISP_AUDIO);
        }
    }

    // ●JPEGの種別によって音声情報を加算するかどうか決める●
    minidx = maxidx;
    maxidx += CAM_TYPENUM_JPEG;
    for (; kindidx < maxidx; kindidx++) {
        // BB-HCM5,3,1以外は音声サイズを加算する
        if (camKind_Jpeg_Hcm5 != gCamModelNameUI[kindidx]) {
            // 計算用ビットレート加算
            gCalcMbps[kindidx] += Number(CALC_AUDIO) + Number(AUDIOADDDATA);
            gCalcJpegMbps[kindidx - minidx] +=
                Number(CALC_AUDIO) + Number(AUDIOADDDATA);
            // 表示用ビットレート加算
            gDispMbps[kindidx] += Number(DISP_AUDIO);
        }
    }

    // ●MPEG4はすべて音声対応●
    minidx = maxidx;
    maxidx += CAM_TYPENUM_MPG4;
    for (; kindidx < maxidx; kindidx++) {
        // 計算用ビットレート加算
        gCalcMbps[kindidx] += Number(CALC_AUDIO) + Number(AUDIOADDDATA);
        // 表示用ビットレート加算
        gDispMbps[kindidx] += Number(DISP_AUDIO);
    }

    // ●H.264(顔照合)はすべて音声対応●
    minidx = maxidx;
    maxidx += CAM_TYPENUM_FACE;
    for (; kindidx < maxidx; kindidx++) {
        // 計算用ビットレート加算
        gCalcMbps[kindidx] += Number(CALC_AUDIO) + Number(AUDIOADDDATA);
        // 表示用ビットレート加算
        gDispMbps[kindidx] += Number(DISP_AUDIO);
    }
}

/* ***************************************************************************
    型番情報をビットレートでソートする(昇順)

    rate1    型番情報1
    rate2    型番情報2

    0 >      model1のビットレートが大きい
    0        同じ
    0 <      model2のビットレートが大きい
**************************************************************************** */
function SortEachModelInfo_Aorder(model1, model2) {
    // ビットレート取得
    var product_id1 = model1['camCalcBR'];
    var product_id2 = model2['camCalcBR'];

    // 値を比較
    return product_id1 - product_id2;
}

/* ***************************************************************************
    型番情報をビットレートでソートする(降順)

    rate1    型番情報1
    rate2    型番情報2

    0 >      model2のビットレートが大きい
    0        同じ
    0 <      model1のビットレートが大きい
**************************************************************************** */
function SortEachModelInfo_Dorder(model1, model2) {

    // ビットレート取得
    var product_id1 = model1['camCalcBR'];
    var product_id2 = model2['camCalcBR'];

    // 値を比較
    return product_id2 - product_id1;
}

/* ***************************************************************************
    NWDRの録画情報を計算する

    nNwdrCode 0 : ND400
              4 : NV300
    return    1 : 正常終了
             -1 : 計算不可
**************************************************************************** */
function basicNwdrCalc(nNwdrCode) {
    // 仮に割り当てるNWDR台数を決定
    // この関数の中で、カメラ情報をビットレート降順にソートしている
    var tmpRecNum = getTempNwdrNum(nNwdrCode);

    // 顔照合カメラがある場合は、少なくとも、すべての顔照合カメラを収容可能なNWDRが必要
    if (tmpRecNum < Math.ceil(gCamNumberUI[H264FM] / defMaxFMcamNum)) {
        tmpRecNum = Math.ceil(gCamNumberUI[H264FM] / defMaxFMcamNum);
    }

    // 最初から1台も割り当て出来ない場合は、
    // 計算するまでもない
    if (tmpRecNum < 1) {
        return -1;
    }

    // ===============================================================
    // ◆◆◆降順ソートによる割り当て(1st)◆◆◆
    // ===============================================================
    // ■■■STEP6■■■
    // 本体を増やす事で、増設が減るパターンがあるのでチェックする
    var plusRecNum = tmpRecNum;
    for (var idx = 0; idx < defPlusRecMax; idx++)
    {
        // 再計算するので、計算結果を保存しておく
        var orgNwdrInfo = gNwdrInfo.clone();
        // レコーダー情報初期化
        NWDRArrayInit(plusRecNum);
        while (true) {
            // ■■■STEP5-2■■■(STEP3～STEP4)
            // 割り当て実施
            var buf = AssignCam2Nwdr(nNwdrCode, plusRecNum);
            if (0 > buf) {
                return buf; // 割り当てられなかった
            } else if (1 == buf) { // 容量不足
                plusRecNum++; // レコーダーを1台追加
                NWDRArrayInit(plusRecNum);
            } else { // 割当成功
                break;
            }
        }

        if (0 != idx) { // 最初の計算は比較不要
            // レコーダ台数変更前後でコストの低い側を残す
            FunCompareOptimum(gNwdrInfo, orgNwdrInfo, nNwdrCode);
        }
        plusRecNum++; // 本体を+1して再計算
    }

    // 再計算するので、1回目の計算結果を保存しておく
    var copyNwdrInfo = gNwdrInfo.clone();

    // ===============================================================
    // ◆◆◆昇順ソートによる割り当て(2nd)◆◆◆
    // ===============================================================
    // ■■■STEP5-1■■■
    // カメラ情報をソートする(ビットレート昇順)
    gCamModelParamInfo.sort(SortEachModelInfo_Aorder);
    // ■■■STEP6■■■
    plusRecNum = tmpRecNum;
    for (var idx = 0; idx < defPlusRecMax; idx++)
    {    // 本体を増やす事で、増設が減るパターンがあるのでチェックする
        var orgNwdrInfo = gNwdrInfo.clone(); // 再計算のため計算結果を保存
        // レコーダー情報初期化
        NWDRArrayInit(plusRecNum);
        while (true) {
            // ■■■STEP5-2■■■(STEP3～STEP4)
            // 割り当て実施
            var buf = AssignCam2Nwdr(nNwdrCode, plusRecNum);
            if (0 > buf) {
                return buf; // 割り当てられなかった
            } else if (1 == buf) { // 容量不足
                plusRecNum++; // レコーダーを1台追加
                NWDRArrayInit(plusRecNum);
            } else { // 割当成功
                break;
            }
        }

        if (0 != idx) { // 最初の計算は比較不要
            // レコーダ台数変更前後でコストの低い側を残す
            FunCompareOptimum(gNwdrInfo, orgNwdrInfo, nNwdrCode);
        }
        plusRecNum++; // 本体を+1して再計算
    }

    // ■■■STEP5-3■■■
    // ソートする前と後で、コストの低い側を残す
    FunCompareOptimum(gNwdrInfo, copyNwdrInfo, nNwdrCode);

    // 顔照合カメラを複数台割り当てようとした場合は、
    // ここで終了
    if (g_FaceOverFlag) {
        return 1;
    }

    return 1;
}

/* ***************************************************************************
    2つのNWDR情報を比較し、金額的に安い側を選択する
    (価格比率固定)

    NwdrInfo1    NWDR情報1
    NwdrInfo2    NWDR情報2
    nNwdrCode    NWDR機種コード

    1            NWDR情報1の方が高い
    0            NWDR情報2の方が高い
**************************************************************************** */
function FunCompareOptimum(NwdrInfo1, NwdrInfo2, nNwdrCode) {
    var NwdrIdx1 = 0; var NwdrIdx2 = 0;
    var ExtUnitNum1 = 0; var ExtUnitNum2 = 0;

    NwdrIdx1 = NwdrInfo1.length;
    NwdrIdx2 = NwdrInfo2.length;

    // 増設ユニット数のカウント
    for (var i = 0; i < NwdrIdx1; i++) {
        ExtUnitNum1 = Number(ExtUnitNum1) + Number(NwdrInfo1[i].extUnitNum);
    }
    for (var i = 0; i < NwdrIdx2; i++) {
        ExtUnitNum2 = Number(ExtUnitNum2) + Number(NwdrInfo2[i].extUnitNum);
    }
    var lratio = Number(defValRatio[nNwdrCode]) * (NwdrIdx1 - NwdrIdx2);

    if ((ExtUnitNum2 - ExtUnitNum1) >= lratio) {
        return 1;
    }

    tempnum = NwdrInfo2.length;
    NWDRArrayInit(tempnum);
    gNwdrInfo = NwdrInfo2.clone();
    return 0;
}

/* ***************************************************************************
    1セット(レコーダーor 推奨サーバ+増設Max)に収まり切らない容量のカメラが
    あるかチェックする

    nNwdrCode 0 : ND400
              4 : NV300
**************************************************************************** */
function FunCheckOverData(nNwdrCode) {
    // 増設容量のみ計算
    gMaxDataCapTotalTB = Number(gMaxDataCapExtGB) * Number(defLimitOfExtNum[nNwdrCode]) / 1000;
    // 続いて本体を足しこむ(本体＋増設)
    gMaxDataCapTotalTB = Number(gMaxDataCapTotalTB) + (Number(gMaxDataCapMainGB) / 1000);

    // カメラ種別毎に調べる
    for (var i = 0; i < gCamModelParamInfo.length; i++) {
        var buf = gCamModelParamInfo[i]; // カメラ種別
        if (buf['camNum'] != 0) { // 台数が入力されているもののみチェック
            // 収まりきらないか
            if ((Number(buf['camRecTB']) / Number(defRecCoef[nNwdrCode])) > gMaxDataCapTotalTB) {
                return RET_NWDR_OVF;
            }
        }
    }
    return RET_NO_ERROR;
}

/* ***************************************************************************
    NWDRにカメラを割り当てる(メイン)

    tempnum  仮割り当てしたNWDR台数
    nNwdrCode 0 : ND400
              4 : NV300
**************************************************************************** */
function AssignCam2Nwdr(nNwdrCode, tempnum) {

    var intNWDRq = 0; // 割当中のNWDR番号

    // カメラ割当処理済管理(初期値=false) // 2028_62321
    var bCamHandled = new Array(gTotalCamNum + 1);
    for (var camno = 0; camno < gTotalCamNum + 1; camno++) {
        bCamHandled[camno] = false;
    }

    // 1セット(レコーダーor 推奨サーバ+増設Max)に収まり切らない容量のカメラがあった場合は、
    // 計算するまでもない
    var bigcheck = FunCheckOverData(nNwdrCode);
    if (bigcheck < 0) {
        return -1;
    }

    // =======================
    // 初期設定
    // =======================
    // NWDR「本体」1台あたりのユニット容量[TB]
    var oneMainUnitCapaTB = Number(gMaxDataCapMainGB) / 1000;

    var bMainUnitFinishedFlag = false; // NWDR本体割当終了フラグ
    var bHbrCamFinishedFlag = false; // 高ビットレートカメラ割当終了フラグ

    // =======================================================================
    // カメラ割り当て(1)◆顔照合◆
    // =======================================================================
    for (var camno = 1; camno < gTotalCamNum + 1; camno++) { // 全カメラをサーチ
        if (bCamHandled[camno] == true) continue; // 割当済カメラはスキップする

        // カメラ情報取得
        targetcaminfo = GetSingleCamInfo(nNwdrCode, camno);
        if (targetcaminfo[CAM_TYPE] != H264FM) continue;  // 顔照合カメラ以外なら次のカメラを探す

        // 割当可能なNWDRを探す
        for (var nwdrno = intNWDRq; nwdrno < tempnum; nwdrno++) {
            var ret = SubChangeNwdrChecklimit(nNwdrCode, targetcaminfo, nwdrno, 2); // 2:容量チェックをスキップ
            if (0 > ret) { // 1台目となる顔照合すら割り当てられない場合は
                return -1; // そのまま、error return
            } else {
                if (ret == 1) { // 本体に入らない場合は RAID5/6 のケースなので、即、増設を行う
                    addExtUnit(nwdrno);
                }
                // カメラを対象NWDRへ登録
                RegCam2Nwdr(nwdrno,            // NWDR番号
                                targetcaminfo[CAM_CALCBR],  // ビットレート
                                targetcaminfo[CAM_TYPE],    // カメラ種別
                                targetcaminfo[CAM_JPGIPS],  // JPEG IPS
                                targetcaminfo[CAM_RECTB],   // 録画データサイズ
                                targetcaminfo[CAM_DISPBR],  // 表示ビットレート
                                targetcaminfo[CAM_MODEL],  // 型番選択情報
                                targetcaminfo[CAM_DISPVBR]); // 表示用映像ビットレート

                // 最終HDD割当容量+カメラ追加[TB]
                gNwdrInfo[nwdrno].lngLastHDDCapa =
                    Number(gNwdrInfo[nwdrno].lngLastHDDCapa) + Number(targetcaminfo[CAM_RECTB]);

                // 次のカメラは、今割り当てた次のNWDRから割り当てる
                intNWDRq++;
                if (intNWDRq >= tempnum) {
                    intNWDRq = 0;
                }
                // 割当終了
                bCamHandled[camno] = true;
                break;
            }
        }
    }

    // =======================================================================
    // カメラ割り当て(2)◆高ビットレート◆
    // =======================================================================
    if (nNwdrCode == nNwdrND400) { // ND400のみ対象
        intNWDRq = 0; // 割当NWDRを先頭に戻す
        // --------------------------
        // まずは、本体側で割り当てる
        // --------------------------
        for (var camno = 1; camno < gTotalCamNum + 1; camno++) {
            if (bCamHandled[camno] == true) continue; // 割当済カメラはスキップする

            // カメラ情報取得
            targetcaminfo = GetSingleCamInfo(nNwdrCode, camno);

            // 高ビットレートカメラのみ抽出
            // 表示用ビットレートと比較する
            if (targetcaminfo[CAM_DISPVBR] <= ND400_NORMAL_LIM_MBPS)
            {
                // 高ビットレートカメラ以外なら次のカメラを探す
                continue;
            }

            // 本体に収まる容量か
            if (targetcaminfo[CAM_RECTB] > oneMainUnitCapaTB) {
                break; // 本体割当ループを抜けて、増設を使う様にする
            }

            // ■■■STEP3-2■■■
            // 割当可能なNWDRを探す
            for (var nwdrno = intNWDRq; nwdrno < tempnum; nwdrno++) {
                // 本体のみで割当可能か(割当不可でも、他のNWDRは探さない)
                var ret = SubChangeNwdrChecklimit(nNwdrCode, targetcaminfo, nwdrno, 0); // 0:本体のみ
                if (ret < 0) {
                    // 終了フラグを立てる
                    bHbrCamFinishedFlag = true;
                    break;
                } else {
                    // カメラを対象NWDRへ登録
                    RegCam2Nwdr(nwdrno,            // NWDR番号
                                targetcaminfo[CAM_CALCBR], // ビットレート
                                targetcaminfo[CAM_TYPE],   // カメラ種別
                                targetcaminfo[CAM_JPGIPS], // JPEG IPS
                                targetcaminfo[CAM_RECTB],  // 録画データサイズ
                                targetcaminfo[CAM_DISPBR], // 表示ビットレート
                                targetcaminfo[CAM_MODEL], // 型番選択情報
                                targetcaminfo[CAM_DISPVBR]); // 表示用映像ビットレート

                    // 最終HDD割当容量+カメラ追加[TB]
                    gNwdrInfo[nwdrno].lngLastHDDCapa =
                        Number(gNwdrInfo[nwdrno].lngLastHDDCapa) +
                        Number(targetcaminfo[CAM_RECTB]);

                    // 次のカメラは、今割り当てた次のNWDRから割り当てる
                    intNWDRq++;
                    if (intNWDRq >= tempnum) {
                        intNWDRq = 0;
                    }
                    // 割当終了
                    bCamHandled[camno] = true;
                    break;
                }
            }

            // ■■■STEP3-4■■■
            // 終了フラグがたっていたら、増設に移る
            if (true == bHbrCamFinishedFlag) {
                break;
            }
        }

        // --------------------------
        // 続いて、増設込で割り当てる
        // --------------------------
        intNWDRq = 0; // 割当NWDRを先頭に戻す

        for (var camno = 1; camno < gTotalCamNum + 1; camno++) { // 2028_62321
            if (bCamHandled[camno] == true) continue; // 割当済カメラはスキップする

            // カメラ情報取得
            targetcaminfo = GetSingleCamInfo(nNwdrCode, camno);

            // 高ビットレートカメラのみ抽出
            // 表示用ビットレートと比較する
            if (targetcaminfo[CAM_DISPVBR] <= ND400_NORMAL_LIM_MBPS)
            {
                // 高ビットレートカメラ以外なら次のカメラを探す
                continue;
            }

            // ■■■STEP4-1■■■
            // 増設含めて割当可能か(割当不可の場合は、return 1 -> NWDR本体を増設)
            intNWDRq = AddCamtoAnotherUnit(nNwdrCode, targetcaminfo, intNWDRq, 1); // 1:増設あり
            if (intNWDRq < 0) {
                return 1;
            }
            // カメラを対象NWDRへ登録
            RegCam2Nwdr(intNWDRq,                  // NWDR番号
                        targetcaminfo[CAM_CALCBR], // ビットレート
                        targetcaminfo[CAM_TYPE],   // カメラ種別
                        targetcaminfo[CAM_JPGIPS], // JPEG IPS
                        targetcaminfo[CAM_RECTB],  // 録画データサイズ
                        targetcaminfo[CAM_DISPBR], // 表示ビットレート
                        targetcaminfo[CAM_MODEL], // 型番選択情報
                    targetcaminfo[CAM_DISPVBR]); // 表示用映像ビットレート

            bCamHandled[camno] = true; // 割当終了
        }
    }
    // =======================================================================
    // カメラ割り当て(3)◆一般カメラ◆
    // =======================================================================
    intNWDRq = 0; // 割当NWDRを先頭に戻す
    // --------------------------
    // まずは、本体側で割り当てる
    // --------------------------
    for (var camno = 1; camno < gTotalCamNum + 1; camno++) {
        if (bCamHandled[camno] == true) continue; // 割当済カメラはスキップする

        // カメラ情報取得
        targetcaminfo = GetSingleCamInfo(nNwdrCode, camno);

        // 本体に収まる容量か
        if (targetcaminfo[CAM_RECTB] > oneMainUnitCapaTB) {
            break; // 本体割当ループを抜けて、増設を使う様にする
        }

        // ■■■STEP3-2■■■
        // 割当可能なNWDRを探す
        for (var nwdrno = intNWDRq; nwdrno < tempnum; nwdrno++) {
            // 本体のみで割当可能か(割当不可でも、他のNWDRは探さない)
            var ret = SubChangeNwdrChecklimit(nNwdrCode, targetcaminfo, nwdrno, 0); // 0:本体のみ
            if (0 > ret) {
                // このNWDRには割当不可
                // 最終NWDRでも割当可能なものがなかったら、増設に移る
                if (nwdrno == tempnum - 1) {
                    bMainUnitFinishedFlag = true; // 終了フラグを立てる
                    break;
                }
            }
            else {
                // カメラを対象NWDRへ登録
                RegCam2Nwdr(nwdrno,            // NWDR番号
                            targetcaminfo[CAM_CALCBR], // ビットレート
                            targetcaminfo[CAM_TYPE],   // カメラ種別
                            targetcaminfo[CAM_JPGIPS], // JPEG IPS
                            targetcaminfo[CAM_RECTB],  // 録画データサイズ
                            targetcaminfo[CAM_DISPBR], // 表示ビットレート
                            targetcaminfo[CAM_MODEL], // 型番選択情報
                            targetcaminfo[CAM_DISPVBR]); // 表示用映像ビットレート

                // 最終HDD割当容量+カメラ追加[TB]
                gNwdrInfo[nwdrno].lngLastHDDCapa =
                    Number(gNwdrInfo[nwdrno].lngLastHDDCapa) +
                    Number(targetcaminfo[CAM_RECTB]);

                // 次のカメラは、今割り当てた次のNWDRから割り当てる
                intNWDRq++;
                if (intNWDRq >= tempnum) {
                    intNWDRq = 0;
                }
                // 割当終了
                bCamHandled[camno] = true;
                break;
            }
        }

        // ■■■STEP3-4■■■
        // 終了フラグがたっていたら、増設に移る
        if (true == bMainUnitFinishedFlag) {
            break;
        }
    }

    // --------------------------
    // 続いて、増設込で割り当てる
    // --------------------------
    intNWDRq = 0; // 割当NWDRを先頭に戻す

    for (var camno = 1; camno < gTotalCamNum + 1; camno++) { // 2028_62321
        if (bCamHandled[camno] == true) continue; // 割当済カメラはスキップする

        // カメラ情報取得
        targetcaminfo = GetSingleCamInfo(nNwdrCode, camno);
        if (targetcaminfo[CAM_TYPE] == H264FM) continue;  // 顔照合カメラなら、割当済みなので次のカメラを探す

        // ■■■STEP4-1■■■
        // 増設含めて割当可能か(割当不可の場合は、return 1 -> NWDR本体を増設)
        intNWDRq = AddCamtoAnotherUnit(nNwdrCode, targetcaminfo, intNWDRq, 1); // 1:増設あり
        if (intNWDRq < 0) {
            return 1;
        }
        // カメラを対象NWDRへ登録
        RegCam2Nwdr(intNWDRq,                  // NWDR番号
                    targetcaminfo[CAM_CALCBR], // ビットレート
                    targetcaminfo[CAM_TYPE],   // カメラ種別
                    targetcaminfo[CAM_JPGIPS], // JPEG IPS
                    targetcaminfo[CAM_RECTB],  // 録画データサイズ
                    targetcaminfo[CAM_DISPBR], // 表示ビットレート
                    targetcaminfo[CAM_MODEL], // 型番選択情報
                    targetcaminfo[CAM_DISPVBR]); // 表示用映像ビットレート
        bCamHandled[camno] = true; // 割当終了
    }

    return RET_NO_ERROR;
}

/* ***************************************************************************
    今からカメラを割当しようとしているNWDRが、
    性能上限を超えないかどうかをチェックする

    hdeflag      0 : 本体容量チェック/容量不足->error
                 1 : 本体＋増設での容量チェック/容量不足->error
                 2 : 本体のみの容量チェック/容量不足も正常扱い // 2028_62321

    return  RET_NO_ERROR : カメラ割当可能
                     -1    カメラ割当不可
                      1    増設側でカメラ割当可 // 2028_62321
**************************************************************************** */
function SubChangeNwdrChecklimit(nNwdrCode, targetcaminfo, intNWDRq, hdeflag) {

    // 既に割当不可と判断されている場合は見るまでもなく割当不可
    if (1 < gNwdrInfo[intNWDRq].pankflg) { // 本体割り当て不可なら
        return -1;
    }

    // カメラのビットレートを算出する
    var camBitrate = 0;

    // JPEGの場合は defGetaJpeg を処理
    if (JPEG_1 <= targetcaminfo[CAM_TYPE][0] && targetcaminfo[CAM_TYPE][0] <= JPEG_3) {
        // カメラ設定よりビットレートが高くなるので、その係数をかける
        camBitrate = Number(targetcaminfo[CAM_CALCBR]) * Number(defGetaJpeg);
    } else { // その他のカメラの場合
        camBitrate = Number(targetcaminfo[CAM_CALCBR]); // そのままのビットレート
    }

    // ----------------------------------
    // ◆ビットレート上限チェック◆
    // ----------------------------------
    // H.264カメラは、AVBR対応により表示用Bitrate値を用いる
    var nextBitrateMbps =
        Number(gNwdrInfo[intNWDRq].nCamNumH264_1) * gVideoMbps[H264_1] + //H.264(1)
        Number(gNwdrInfo[intNWDRq].nCamNumH264_2) * gVideoMbps[H264_2] + //H.264(2)
        Number(gNwdrInfo[intNWDRq].nCamNumH264_3) * gVideoMbps[H264_3] + //H.264(3)
        Number(gNwdrInfo[intNWDRq].nCamNumJpeg_1) * gDispMbps[JPEG_1] / 1000 * 1024 + //JPEG(1)
        Number(gNwdrInfo[intNWDRq].nCamNumJpeg_2) * gDispMbps[JPEG_2] / 1000 * 1024 + //JPEG(2)
        Number(gNwdrInfo[intNWDRq].nCamNumJpeg_3) * gDispMbps[JPEG_3] / 1000 * 1024 + //JPEG(3)
        Number(gNwdrInfo[intNWDRq].nCamNumMpg4_1) * gVideoMbps[MPG4_1] + //MPEG4
        Number(gNwdrInfo[intNWDRq].nCamNumFace_1) * gVideoMbps[H264FM] + //H.264(顔照合)
        gVideoMbps[targetcaminfo[CAM_TYPE][0]];

    // ■■■STEP2-2■■■
    // ----------------------------------
    // ◆ND400 HBRカメラ台数上限チェック◆
    // ----------------------------------
    if (nNwdrCode == nNwdrND400) {
        if (Number(targetcaminfo[CAM_DISPVBR]) > ND400_NORMAL_LIM_MBPS) {
            if (Number(gNwdrInfo[intNWDRq].lngHbrCamNum) >= ND400_HBRCAM_MAX) {
                if(gNwdrInfo[intNWDRq].pankflg == 0) { // ND400のみ対象
                    gNwdrInfo[intNWDRq].pankflg = 1;  // このNWDRへは割り当て不可
                }
                return RET_HBRCAM_OVF;
            }
        }
    }

    if (defLimitBandWidth[nNwdrCode] < nextBitrateMbps) {
        gNwdrInfo[intNWDRq].pankflg = 2; // このNWDRへは割り当て不可
        return RET_NWDR_OVF;
    }

    // ----------------------------------
    // ◆接続数上限チェック◆
    // ----------------------------------
    // ■■■STEP2-1■■■
    // 接続数が性能上限を超えていないか
    var nextMicNum = ((camKind_H264_360SMR == Number(targetcaminfo[CAM_MODEL])) || (camKind_H264_9MSMR == Number(targetcaminfo[CAM_MODEL])) ? 1 : 0);
    var nextCamNum = Number(gNwdrInfo[intNWDRq].nCamTotalNum) +
                     Number(gNwdrInfo[intNWDRq].n360MicNum) +
                     nextMicNum;
    if (Number(g_DynaConnect[nNwdrCode]) <= nextCamNum) { // カメラ＋マイク総数
        gNwdrInfo[intNWDRq].pankflg = 2;  // このNWDRへは割り当て不可
        return RET_NWDR_OVF;
    }

    // ----------------------------------
    // ◆ips値上限チェック◆
    // ----------------------------------
    // ■■■STEP2-4■■■
    // JPEGのfps数が性能上限を超えていないか
    var nextJpegFps =
        Number(gNwdrInfo[intNWDRq].sysTotalIps) + Number(targetcaminfo[CAM_JPGIPS]);
    if (nextJpegFps > Number(defMaxJpegIps[nNwdrCode])) {
        gNwdrInfo[intNWDRq].pankflg = 2; // このNWDRへは割り当て不可
        return RET_NWDR_OVF;
    }

    // ----------------------------------
    // ◆データ容量チェック◆
    // ----------------------------------
    // ■■■STEP2-3■■■
    // 容量チェック (タイムラインについては gMaxDataCapMainGB 側で考慮済)
    var nextRecCapa1 =
        Number(gNwdrInfo[intNWDRq].sysRecDataTB) + Number(targetcaminfo[CAM_RECTB]); // 総容量(NWDR録画済容量)＋追加容量[TB]

    var nextRecCapa2 = 0;
    if (0 == Number(gNwdrInfo[intNWDRq].extUnitNum)) { // 増設ユニット台数=0なら
        nextRecCapa2 =
            Number(gNwdrInfo[intNWDRq].lngLastHDDCapa) + Number(targetcaminfo[CAM_RECTB]); // 最終(本体or増設)HDD割当容量+カメラ追加[TB]
    } else {
        nextRecCapa2 =
            (gMaxDataCapMainGB / 1000) +     // 本体容量
            Number(gNwdrInfo[intNWDRq].extUnitNum - 1) * (gMaxDataCapExtGB / 1000) + // 増設ユニット容量(full)
            Number(gNwdrInfo[intNWDRq].lngLastHDDCapa) + // 最終(本体or増設)HDD割当容量
            Number(targetcaminfo[CAM_RECTB]); //カメラ追加[TB]
    }

    var maxDataCapaTB = 0;
    switch (hdeflag) {
        case 0: // 本体のみで計算
        case 2: // 本体のみで計算、強制増設へ(RAID5/6対策)
            maxDataCapaTB = gMaxDataCapMainGB / 1000;
            break;
        case 1: // 本体＋増設で計算
            maxDataCapaTB = gMaxDataCapTotalTB;
            break;
    }

    if (maxDataCapaTB < nextRecCapa1 || maxDataCapaTB < nextRecCapa2) {
        // この場合、別のカメラなら割当出来るかもしれないので
        // フラグ(pankflg)はそのままにしておく
        return ((hdeflag == 2) ? 1 : RET_NWDR_OVF); // RAID5/6 顔照合対応 // 2028_62321
    }

    return RET_NO_ERROR; // ここまで到達した場合は、割当できます
}

/* ***************************************************************************
    (複数ある)NWDRのなかで、カメラが割当可能なNWDR番号を調べる
    もし割当不可だった場合は、別のNWDRで空いているものを調べる

    return -1   どのNWDRにも割当出来なかった
    それ以外    割当可能なNWDR番号
**************************************************************************** */
function SubChangeNwdrCheck(nNwdrCode, targetcaminfo, intNWDRq, hdeflag) {

    // まず、指定されたNWDRに割当可能かどうか調べる
    var ret = SubChangeNwdrChecklimit(nNwdrCode, targetcaminfo, intNWDRq, hdeflag);
    if (0 == ret) { // 割当可能だった
        return intNWDRq; // NWDRの番号を返す
    }

    // 割当出来なかったので、他のNWDRを探す
    for (var idx = 0; idx < gNwdrInfo.length; idx++) {
        // 割当可能か
        ret = SubChangeNwdrChecklimit(nNwdrCode, targetcaminfo, idx, hdeflag);
        if (0 == ret) { // 割当可能だった
            return idx; // NWDRの番号を返す
        }
    }

    return RET_NWDR_OVF; // 割当可能なNWDRは1台もなかった
}


/* ***************************************************************************
    NWDR本体 or 増設にカメラが割当可能かどうか調べる
    もし割当可能だが、今使用中の本体、もしくは増設ユニットで容量が
    不足する場合は増設ユニットの数を増やす

    return -1   どのNWDRにも割当出来なかった
    それ以外    割当可能なNWDR
**************************************************************************** */
function AddCamtoAnotherUnit(nNwdrCode, targetcaminfo, intNWDRq, hdeflag) {
    var targetUnitMaxCapaGB;

    // 割当可能なNWDRが存在するか
    var ret = SubChangeNwdrCheck(nNwdrCode, targetcaminfo, intNWDRq, hdeflag);
    if (0 > ret) { // 割当出来なかった
        return RET_NWDR_OVF;
    }

    // 今使用中の本体、もしくは増設で容量が足りるか
    var nextRecCapa = Number(gNwdrInfo[ret].lngLastHDDCapa) + Number(targetcaminfo[CAM_RECTB]); // 最終(本体or増設)HDD割当容量+カメラ追加[TB]

    // ■■■STEP4-1■■■ 本体のサーチ
    targetUnitMaxCapaGB = (0 == Number(gNwdrInfo[ret].extUnitNum) ? gMaxDataCapMainGB : gMaxDataCapExtGB);
    // ■■■STEP4-1■■■
    if (nextRecCapa > (targetUnitMaxCapaGB / 1000)) { // 足りないときは、次のNWDRを探す
        intNWDRq++;
        if (intNWDRq >= gNwdrInfo.length) {
            intNWDRq = 0;
        }
        // 割当可能なNWDRが存在するか
        ret = SubChangeNwdrCheck(nNwdrCode, targetcaminfo, intNWDRq, hdeflag);
        if (0 > ret) { // 割当出来なかった
            return RET_NWDR_OVF;
        }
    }

    // ■■■STEP4-2■■■
    // 今使用中の本体、もしくは増設で容量が足りるか
    nextRecCapa = Number(gNwdrInfo[ret].lngLastHDDCapa) + Number(targetcaminfo[CAM_RECTB]); // 最終(本体or増設)HDD割当容量+カメラ追加[TB]
    // 比較対象が本体か、増設かで最大収容量が異なる
    targetUnitMaxCapaGB = (0 == Number(gNwdrInfo[ret].extUnitNum) ? gMaxDataCapMainGB : gMaxDataCapExtGB);

    // 対象本体 or 増設ユニットの容量に入らない場合 ...
    if (nextRecCapa > (targetUnitMaxCapaGB / 1000)) {
        // 増設ユニットを増やす事が出来るか
        if (gNwdrInfo[ret].extUnitNum >= defLimitOfExtNum[nNwdrCode]) { // 増設ユニット台数確認
            return RET_EXT_OVF; // もうMaxまで増設をつなげている
        }

        // 次の増設ユニットが必要だ
        gNwdrInfo[ret].lngLastHDDCapa = Number(gNwdrInfo[ret].lngLastHDDCapa) + Number(targetcaminfo[CAM_RECTB]); // 最終(本体or増設)HDD割当容量+カメラ追加[TB]
        var dobhanpadata = addExtUnit(ret);

        // さらに増設ユニットが必要か
        // 本体使用中か、増設使用中かで最大収容量が異なる
        targetUnitMaxCapaGB = (0 == Number(gNwdrInfo[ret].extUnitNum) ? gMaxDataCapMainGB : gMaxDataCapExtGB);
        while (dobhanpadata > targetUnitMaxCapaGB / 1000) {
            // 増設ユニットを増やす事が出来るか
            if (gNwdrInfo[ret].extUnitNum >= defLimitOfExtNum[nNwdrCode]) { // 増設ユニット台数確認
                return RET_EXT_OVF; // もうMaxまで増設をつなげている
            }

            // 増設追加
            dobhanpadata = addExtUnit(ret);
            gNwdrInfo[ret].lngLastHDDCapa = dobhanpadata; // 最終(本体or増設)HDD割当容量
        }
    } else {
        // 今使用中の本体、もしくは増設で足りた
        gNwdrInfo[ret].lngLastHDDCapa = nextRecCapa; // 最終(本体or増設)HDD割当容量
    }

    return ret;
}

/* ***************************************************************************
    NWDR情報に増設ユニットを追加する

    intNWDRq       NWDR番号
**************************************************************************** */
function addExtUnit(intNWDRq) {
    var dobhanpadata = 0;

    // NV300(本体or増設で容量が異なる)の対応 2014.09.05 H.Sasaki
    var targetUnitMaxCapaGB; // 録画先本体もしくは増設ユニットの許容する最大容量[GB]
    targetUnitMaxCapaGB = (0 == Number(gNwdrInfo[intNWDRq].extUnitNum) ? gMaxDataCapMainGB : gMaxDataCapExtGB);

    dobhanpadata = Number(gNwdrInfo[intNWDRq].lngLastHDDCapa) - (targetUnitMaxCapaGB / 1000); // 最終(本体or増設)HDD割当容量
    gNwdrInfo[intNWDRq].extUnitNum = Number(gNwdrInfo[intNWDRq].extUnitNum) + 1; // 増設ユニット台数をインクリメント
    gNwdrInfo[intNWDRq].lngLastHDDCapa = dobhanpadata; // 最終(本体or増設)HDD割当容量
    return dobhanpadata;
}

/* ***************************************************************************
    対象NWDRにカメラを登録する

    i           対象NWDR番号
    brCalcMbps  ビットレート[Mbps]
    cType       カメラ種別
    ips         JPEG IPS
    camDataTB   録画サイズ
    brDispMbps  表示ビットレート
    modelname   型番情報
**************************************************************************** */
function RegCam2Nwdr(i, brCalcMbps, cType, ips, camDataTB, brDispMbps, modelname, brDispVMbps) {
    // 音声・VMD情報を除去
    // 本来は音声ついていないものもあるが、従来の方法を踏襲する
    //6KBカット
    if (frmNWDR.chkaudio.checked == true) {
        brCalcMbps = brCalcMbps - (Number(CALC_AUDIO) + Number(AUDIOADDDATA));
    }
    if (frmNWDR.chkvmd.checked == true) {
        brCalcMbps = brCalcMbps - Number(CALC_VMDINFO);
    }

    // カメラ種別ごとに台数をカウントする
    switch (cType[0]) {
        // ----------------------------------------------------
        // H.264
        case H264_1:
            gNwdrInfo[i].nCamNumH264_1 = Number(gNwdrInfo[i].nCamNumH264_1) + 1;
            if (camKind_H264_360SMR == modelname || camKind_H264_9MSMR == modelname) { // 2028_63164
                gNwdrInfo[i].n360MicNum++;
            }
            break;
        case H264_2:
            gNwdrInfo[i].nCamNumH264_2 = Number(gNwdrInfo[i].nCamNumH264_2) + 1;
            if (camKind_H264_360SMR == modelname || camKind_H264_9MSMR == modelname) { // 2028_63164
                gNwdrInfo[i].n360MicNum++;
            }
            break;
        case H264_3:
            gNwdrInfo[i].nCamNumH264_3 = Number(gNwdrInfo[i].nCamNumH264_3) + 1;
            if (camKind_H264_360SMR == modelname || camKind_H264_9MSMR == modelname) { // 2028_63164
                gNwdrInfo[i].n360MicNum++;
            }
            break;

        // ----------------------------------------------------
        // JPEG
        case JPEG_1:
            gNwdrInfo[i].nCamNumJpeg_1 = Number(gNwdrInfo[i].nCamNumJpeg_1) + 1;
            brCalcMbps = brCalcMbps * defGetaJpeg;
            break;
        case JPEG_2:
            gNwdrInfo[i].nCamNumJpeg_2 = Number(gNwdrInfo[i].nCamNumJpeg_2) + 1;
            brCalcMbps = brCalcMbps * defGetaJpeg;
            break;
        case JPEG_3:
            gNwdrInfo[i].nCamNumJpeg_3 = Number(gNwdrInfo[i].nCamNumJpeg_3) + 1;
            brCalcMbps = brCalcMbps * defGetaJpeg;
            break;

        // ----------------------------------------------------
        // MPEG4
        case MPG4_1:
            gNwdrInfo[i].nCamNumMpg4_1 = Number(gNwdrInfo[i].nCamNumMpg4_1) + 1;
            break;

        // ----------------------------------------------------
        // H.264(顔照合)
        case H264FM:
            gNwdrInfo[i].nCamNumFace_1 = Number(gNwdrInfo[i].nCamNumFace_1) + 1;
            if (gNwdrInfo[i].nCamNumFace_1 > defMaxFMcamNum) { // 顔照合カメラ台数超過時
                g_FaceOverFlag = 1;
            }
            break;
    }

    // 音声・VMD情報を元に戻す
    //4KB加算
    if (frmNWDR.chkaudio.checked == true) {
        brCalcMbps = brCalcMbps + Number(CALC_AUDIO);
    }
    if (frmNWDR.chkvmd.checked == true) {
        brCalcMbps = brCalcMbps + (Number(CALC_VMDINFO));
    }

    // カメラ台数を合計する
    var tempCamBps = 0;
    gNwdrInfo[i].nCamTotalNum = Number(gNwdrInfo[i].nCamNumH264_1) +
                                Number(gNwdrInfo[i].nCamNumH264_2) +
                                Number(gNwdrInfo[i].nCamNumH264_3) +
                                Number(gNwdrInfo[i].nCamNumJpeg_1) +
                                Number(gNwdrInfo[i].nCamNumJpeg_2) +
                                Number(gNwdrInfo[i].nCamNumJpeg_3) +
                                Number(gNwdrInfo[i].nCamNumMpg4_1) +
                                Number(gNwdrInfo[i].nCamNumFace_1);

    // カメラが高ビットレートの場合、台数を別カウントする
    // 映像のビットレートだけでチェックする
    if (Number(brDispVMbps) > ND400_NORMAL_LIM_MBPS) {
        gNwdrInfo[i].lngHbrCamNum = Number(gNwdrInfo[i].lngHbrCamNum) + 1;
        bND400_4kFirm = true; // 専用ファーム対応
    }
    // 増えたカメラのビットレートを加算する
    gNwdrInfo[i].sysCalcMbps = Number(gNwdrInfo[i].sysCalcMbps) + Number(brCalcMbps);
    tempCamBps = gNwdrInfo[i].sysCalcMbps;
    tempCamBps = NumberToFixed2(tempCamBps, 3);
    gNwdrInfo[i].sysCalcMbps = tempCamBps;

    // 増えたカメラの表示用ビットレートを加算する
    gNwdrInfo[i].sysDispMbps =
        Number(gNwdrInfo[i].sysDispMbps) + Number(brDispMbps); // 表示用総ビットレート[Mbps]

    // 増えたカメラのJPEG録画IPS数を加算する
    gNwdrInfo[i].sysTotalIps = Number(gNwdrInfo[i].sysTotalIps) + Number(ips);

    // 増えたカメラの録画容量を加算する
    camDataTB = GetMarumCapa2(camDataTB);
    gNwdrInfo[i].sysRecDataTB =
        Number(gNwdrInfo[i].sysRecDataTB) + Number(camDataTB); // 総容量(NWDR録画済容量)[TB]
}

/* ***************************************************************************
    台数番目のカメラ情報を取得する

    nNwdrCode 0 : ND400
              4 : NV300

    j        台数番目
**************************************************************************** */
function GetSingleCamInfo(nNwdrCode, j) {
    var lmode;
    var inttempc = 0;

    for (var i = 0; i < gCamModelParamInfo.length; i++) {
        var buf = gCamModelParamInfo[i];

        var inttempc = Number(inttempc) + Number(buf['camNum']);
        if (j <= inttempc) {
            intarrayno = j;

            return [
                Number(buf['camRecTB']) / defRecCoef[nNwdrCode], // 0:CAM_RECTB
                buf['camCalcBR'], // 1:CAM_CALCBR
                buf['camType'],   // 2:CAM_TYPE
                buf['camJpgIps'], // 3:CAM_JPGIPS
                buf['camDispBR'], // 4:CAM_DISPBR
                buf['camModel'],   // 5:CAM_MODEL
                buf['camDispVBR']   // 6:CAM_DISPVBR
            ];
        }
    }
}

/* ***************************************************************************
    仮に割り当てるNWDR台数を決める

    nNwdrCode 0 : ND400
              4 : NV300
    return 正値 : 仮決めNWDRシステム(本体+増設)数
           負値 : エラーコード
**************************************************************************** */
function getTempNwdrNum(nNwdrCode) {
    // 仮のNWDR台数
    var tempNwdrNum = 0;

    //Advanced VBR対応＜未使用?★＞
    var sprio1 = frmNWDR.selPrioH264_1.options[
                frmNWDR.selPrioH264_1.selectedIndex
            ].text;
    var sprio2 = frmNWDR.selPrioH264_2.options[
                frmNWDR.selPrioH264_2.selectedIndex
            ].text;
    var sprio3 = frmNWDR.selPrioH264_3.options[
                frmNWDR.selPrioH264_3.selectedIndex
            ].text;

    // 接続可能セッション数から、仮の台数(tempNwdrNum)を算出する
    var modconnect = gTotalCamNum % g_DynaConnect[nNwdrCode];
    tempNwdrNum = (gTotalCamNum - modconnect) / g_DynaConnect[nNwdrCode];
    if (0 < modconnect) {
        // 割り切れないときは、台数を１台増やす
        tempNwdrNum += 1;
    }

    // 総ビットレートから、仮の台数を算出する
    // 表示されている総ビットレートはKbps / 1024 = Mbps
    // 上限ビットレートはKbps / 1000 = Mpbs

    // H.264カメラは、AVBR対応により表示用Bitrate値を用いる
    var totalBitrateMbps =
            gCamNumberUI[H264_1] * gVideoMbps[H264_1] + //H.264(1)
            gCamNumberUI[H264_2] * gVideoMbps[H264_2] + //H.264(2)
            gCamNumberUI[H264_3] * gVideoMbps[H264_3] + //H.264(3)
            gCamNumberUI[JPEG_1] * gDispMbps[JPEG_1] / 1000 * 1024 + //JPEG(1)
            gCamNumberUI[JPEG_2] * gDispMbps[JPEG_2] / 1000 * 1024 + //JPEG(2)
            gCamNumberUI[JPEG_3] * gDispMbps[JPEG_3] / 1000 * 1024 + //JPEG(3)
            gCamNumberUI[MPG4_1] * gVideoMbps[MPG4_1] + //MPEG4
            gCamNumberUI[H264FM] * gVideoMbps[H264FM]; //H.264(顔照合)

    var calcNwdrNum = Math.ceil(totalBitrateMbps / defLimitBandWidth[nNwdrCode]); // 算出(推定)NWDR台数を仮決め

    // 既に計算済みの台数より多い場合は入れ替える
    if (calcNwdrNum > tempNwdrNum) {
        tempNwdrNum = calcNwdrNum;
    }

    // ■■■STEP2-3■■■
    // NWDRに搭載可能なHDD数から、仮の台数を算出する
    // HDD総容量(管理係数をかけた、実記録可能領域)
    var totalTargetRecDataGB = Number(frmNWDR.txttotalcap.value) / defRecCoef[nNwdrCode]; // 目標録画総容量(TBなので注意)
    // TBをGBに変換
    totalTargetRecDataGB = NumberToFixed2(totalTargetRecDataGB, 4) * 1000;

    // NV300-JP 本体HDD容量変動対応(2028_62938)
    var hddCapaGB = gSelHddUnitGB;
    var nv300MainGB = defNV300HDDcapTB[nNV300_8] * 1000; // 300/8 として計算(2台分込み)
    // 総容量 / HDD1本の容量で、必要なHDD数を計算する
    var maxTotalCapaGB;    //NWDRフル構成時の最大容量[GB] (初期化不要)

    switch (gSelRaidMode) {
        // -----------------------------------------------------
        case 0: // Single
        // -----------------------------------------------------
            // NWDR(本体/増設各々)の総記録可能容量[GB]
            gMaxDataCapMainGB = hddCapaGB * defMaxHddNum_Main_RAID0[nNwdrCode];
            // NV300 国内の例外処理 2028_62938
            if (nNwdrCode == nNwdrNV300 && app_dst == 'ja') {
                gMaxDataCapMainGB = nv300MainGB; // 300/8 として計算
            }
            gMaxDataCapExtGB = hddCapaGB * defMaxHddNum_Ext_RAID0[nNwdrCode];
            // NWDR(本体＋増設)の総記録可能容量[GB] 2028_62938->2028_63418
            maxTotalCapaGB = gMaxDataCapMainGB +
                hddCapaGB * defLimitOfExtNum[nNwdrCode] * defMaxHddNum_Ext_RAID0[nNwdrCode];

            // ◆NV300タイムライン補正◆
            if (nNwdrCode == nNwdrNV300) {
                gMaxDataCapMainGB -= defMaxHddNum_Main_RAID0[nNwdrCode] * (defNVtimelineTB * 1000); // 本体
                gMaxDataCapExtGB -= defMaxHddNum_Ext_RAID0[nNwdrCode] * (defNVtimelineTB * 1000); // 増設(ユニットあたり)
                maxTotalCapaGB -= defMaxHddNum_System_RAID0[nNwdrCode] * (defNVtimelineTB * 1000); // 本体増設フル構成
            }
            break;

        // -----------------------------------------------------
        case 1: // RAID1 または
        case 9: // DUAL
        // -----------------------------------------------------
            // NWDR(本体/増設各々)の総記録可能容量[GB]
            gMaxDataCapMainGB = hddCapaGB * defMaxHddNum_Main_RAID1[nNwdrCode];
            gMaxDataCapExtGB = 0;
            // NWDR(本体＋増設)の総記録可能容量[GB]
            maxTotalCapaGB = hddCapaGB * defMaxHddNum_System_RAID1[nNwdrCode];
            // NV300 国内の例外処理 2028_62938
            if (nNwdrCode == nNwdrNV300 && app_dst == 'ja') {
                gMaxDataCapMainGB = nv300MainGB / 2; // 300/8(1台分) として計算
                maxTotalCapaGB = gMaxDataCapMainGB;  // 300/8 として計算
            }

            // ◆NV300タイムライン補正◆
            if (nNwdrCode == nNwdrNV300) {
                gMaxDataCapMainGB -= defMaxHddNum_Main_RAID1[nNwdrCode] * (defNVtimelineTB * 1000); // 本体(※増設はない)
                maxTotalCapaGB -= defMaxHddNum_System_RAID1[nNwdrCode] * (defNVtimelineTB * 1000); // 本体増設フル構成
            }
            break;

        // -----------------------------------------------------
        case 5: // RAID5
        // -----------------------------------------------------
            // ◆NV300◆
            // NWDR(本体/増設各々)の総記録可能容量[GB]
            gMaxDataCapMainGB = hddCapaGB * defMaxHddNum_Main_RAID5[nNwdrCode];
            gMaxDataCapExtGB = hddCapaGB * defMaxHddNum_Ext_RAID5[nNwdrCode];
            // NWDR(本体＋増設)の総記録可能容量[GB]
            maxTotalCapaGB = hddCapaGB * defMaxHddNum_System_RAID5[nNwdrCode];

            // ◆NV300タイムライン補正◆
            if (nNwdrCode == nNwdrNV300) {
                //gMaxDataCapExtGB -= defMaxHddNum_Ext_RAID5[nNwdrCode] * defNVtimelineTB; // 増設(※本体はない)
                //maxTotalCapaGB -= defMaxHddNum_System_RAID5[nNwdrCode] * defNVtimelineTB; // 本体増設フル構成
                gMaxDataCapExtGB -= (defNVtimelineTB * 1000); // 増設(※本体はない)
                maxTotalCapaGB -= defLimitOfExtNum[nNwdrCode] * (defNVtimelineTB * 1000); // 本体増設フル構成
            }
            break;

        // -----------------------------------------------------
        case 6: // RAID6
        // -----------------------------------------------------
            // ◆ND400/NV300◆
            // NWDR(本体/増設各々)の総記録可能容量[GB]
            gMaxDataCapMainGB = hddCapaGB * defMaxHddNum_Main_RAID6[nNwdrCode];
            gMaxDataCapExtGB = hddCapaGB * defMaxHddNum_Ext_RAID6[nNwdrCode];
            // NWDR(本体＋増設)の総記録可能容量[GB]
            maxTotalCapaGB = hddCapaGB * defMaxHddNum_System_RAID6[nNwdrCode];

            // ◆NV300タイムライン補正◆
            if (nNwdrCode == nNwdrNV300) {
                //gMaxDataCapExtGB -= defMaxHddNum_Ext_RAID6[nNwdrCode] * defNVtimelineTB; // 増設(※本体はない)
                //maxTotalCapaGB -= defMaxHddNum_System_RAID6[nNwdrCode] * defNVtimelineTB; // 本体増設フル構成
                gMaxDataCapExtGB -= (defNVtimelineTB * 1000); // 増設(※本体はない)
                maxTotalCapaGB -= defLimitOfExtNum[nNwdrCode] * (defNVtimelineTB * 1000); // 本体増設フル構成
            }

            break;
    } // end of switch
    // NWDR本体推定台数(=システム数推定台数)
    calcNwdrNum = Math.ceil(totalTargetRecDataGB / maxTotalCapaGB);

    // 既に計算済みの台数より多い場合は入れ替える
    if (calcNwdrNum > tempNwdrNum) {
        tempNwdrNum = calcNwdrNum;
    }

    // ■■■STEP2-4■■■
    // JPEGの録画可能IPS数から、仮のNWDR数を算出する
    // JPEG IPS数合計
    var totalJpegIps = 0;
    totalJpegIps += (gCamNumberUI[JPEG_1] * gJpegIps[JPEG_1]); //JPEG(1)
    totalJpegIps += (gCamNumberUI[JPEG_2] * gJpegIps[JPEG_2]); //JPEG(2)
    totalJpegIps += (gCamNumberUI[JPEG_3] * gJpegIps[JPEG_3]); //JPEG(3)

    // 1台の上限値から何台必要か計算
    var modJpegIps = totalJpegIps % defMaxJpegIps[nNwdrCode];
    calcNwdrNum = (totalJpegIps - modJpegIps) / defMaxJpegIps[nNwdrCode];
    if (modJpegIps > 0) {
        // 割り切れないときは、台数を１台増やす
        calcNwdrNum += 1;
    }

    // 既に計算済みの台数より多い場合は入れ替える
    if (calcNwdrNum > tempNwdrNum) {
        tempNwdrNum = calcNwdrNum;
    }

    // 最大ビットレートでの録画容量、仮のNWDR数を算出する
    // ■■■STEP3-1■■■
    var oneDataGB = GetCamBitrateorder(nNwdrCode);
    // NWDR＋増設の総容量に、カメラ1台の録画が
    // 収まりきらなかったらエラーとする
    if (oneDataGB > maxTotalCapaGB) {
        return RET_NWDR_OVF;
    }

    return tempNwdrNum;
}

/* ***************************************************************************
    カメラ情報をビットレート降順に並び替え、
    最大ビットレート値での録画容量を計算する
    (1台のカメラを複数のNWDRにまたがって録画することは
     できないので、その制限チェックに必要)

    nNwdrCode 0 : ND400
              4 : NV300

    return   最大ビットレート値での録画容量
**************************************************************************** */
function GetCamBitrateorder(nNwdrCode) {
    // 最大ビットレート値
    var tempbit = 0;
    for (var i = 0; i < CAM_TYPENUM_ALL; i++) {
        // チェックは、台数が入力されたカメラのみ
        if (gCamNumberUI[i] > 0) {
            if (gCalcMbps[i] > tempbit) {
                tempbit = gCalcMbps[i];
            }
        }
    }

    // 最大ビットレート値での録画容量
    var oneDataGB = 0;
    oneDataGB =
        Number(tempbit) / 8 * 3600 * gRecDay * gRecHour / 1000;
    oneDataGB = oneDataGB / Number(defRecCoef[nNwdrCode]);


    oneDataGB = Number(NumberToFixed2(oneDataGB, 4));

    // ■■■STEP3-1■■■
    // カメラ情報をソートする(降順)
    gCamModelParamInfo.sort(SortEachModelInfo_Dorder);

    return oneDataGB;
}

/* ***************************************************************************
    比較
**************************************************************************** */
function hikaku(a, b) {
    return (b.Mbps - a.Mbps);
}

/* ***************************************************************************
    カメラをビットレートに関して大小交互にソート
**************************************************************************** */
function sort_kougo(cnt, camBitrate) {
    var cnt_l = 0; //小さい方からカウント
    var cnt_s = 0; //大きい方からカウント

    for (var i = 0; i < cnt; i++) {
        if (i % 2 == 0) {
            tmpcamBitrate[i] = camBitrate[cnt_s];
            cnt_s++;
        }
        else {
            tmpcamBitrate[i] = camBitrate[cnt - 1 - cnt_l];
            cnt_l++;
        }
    }
    for (var i = 0; i < cnt; i++) {
        camBitrate[i] = tmpcamBitrate[i];
    }
}

/* ***************************************************************************
    camNum2LicNum : ライセンス数を求める

    nCamNum  カメラ数
    nNwdrCode 0 : ND400
              4 : NV300
**************************************************************************** */
function camNum2LicNum(nCamNum, nNwdrCode) {
    var nRet = 0;
    switch (nNwdrCode) {
        case nNwdrNV300: //JP and PAL
            if (nCamNum > 24) nRet++;
            if (nCamNum > 16) nRet++;
            if (nCamNum > 9) nRet++;
            break;

    } // end of switch
    return nRet;
}

/* ***************************************************************************
    ブラウザのバージョンを取得する

    @return Number:
**************************************************************************** */
function getBrowserVersion() {
    return window.opera ?
        (opera.version().replace(/\d$/, '') - 0) :
        parseFloat((/(?:IE |fox\/|ome\/|ion\/)(\d+\.\d)/.
            exec(navigator.userAgent) || [, 0])[1]);
}

/* ***************************************************************************
    数値入力のみを許可する

    true     反応する
    false    反応しない
**************************************************************************** */
function numOnly2(nKeyCode) {

    // DEL(46)、左(37)右(39)矢印キーには反応する
    if (nKeyCode == 46 || nKeyCode == 37 || nKeyCode == 39) {
         return true;
    }

    // テンキーの0-9にも反応する
    if (96 <= nKeyCode & nKeyCode <= 105) {
        return true;
    }

    // 文字コードを文字列に変換
    var m = String.fromCharCode(nKeyCode);
    // 数字と改行コード等でなければ反応しない
    if ('0123456789\d\b\r\t'.indexOf(m, 0) < 0) {
         return false;
    }

    return true;

}

/* ***************************************************************************
    数値の丸め処理をおこなう

    小数点4桁以下の場合0になってしまうので補正する
**************************************************************************** */
function GetMarumCapa(data) {

    if (data < 1) {
        if (data < 0.0001) {
            data = 0.0001;
        } else {
            data = Math.ceil(data * 100000) / 100000;
            data = NumberToFixed(data, 4);
        }
    } else {
        data = Math.ceil(data * 1000) / 1000;
        data = NumberToFixed(data, 2);
    }

    return data;
}

/* ***************************************************************************
    数値の丸め処理をおこなう(計算用に丸めをおこなわない)

**************************************************************************** */
function GetMarumCapa2(data) {

    return data;
}

/* ***************************************************************************
    数値を四捨五入する
**************************************************************************** */
function NumberToFixed(dValue, iDigits) {
    // 無効値をチェック
    if (isNaN(dValue)) {
        return dValue;
    }

    var dCoef = Math.pow(10, iDigits);
    var ret = dValue;

    if (dValue > 0) {
        ret = Math.floor((dValue * dCoef) + 0.5) / dCoef;
    }
    else {
        ret = Math.ceil((dValue * dCoef) - 0.5) / dCoef;
    }

    // 数値を文字列に変換
    var retstr = ret.toString();
    var pos = retstr.lastIndexOf('.');
    if (0 < pos) {
        // 今小数点が何桁あるか
        var keta = retstr.length - (pos + 1);
        // 桁数が足りない分だけ0を付ける
        for (var idx = keta; idx < iDigits; idx++) {
            retstr = retstr + '0';
        }
    }
    else {
        retstr = ret.toString();

        // 小数点付与
        retstr = retstr + '.';
        // 桁数が足りない分だけ0を付ける
        for (var idx = 0; idx < iDigits; idx++) {
            retstr = retstr + '0';
        }
    }

    return retstr;
}

/* ***************************************************************************
    数値を四捨五入する(計算用に四捨五入しない)
**************************************************************************** */
function NumberToFixed2(dValue, iDigits) {
    return dValue;
}

/* ***************************************************************************
    JP-Single判定を行う (全機種共通)
**************************************************************************** */
function isJpSingle() {
    if (gSelRaidMode == 0 && app_dst == 'ja') return true;
    return false;
}

/* ***************************************************************************
    JP-Mirroring判定を行う (全機種共通)
**************************************************************************** */
function isJpMirror() {
    if (gSelRaidMode == 1 && app_dst == 'ja') return true;
    return false;
}


